@@ -43,6 +43,8 @@
 typedef struct {
   int seconds;
   struct timeval time;
+  int ripped;
+  int taken;
 } Progress;
 
 /** If this module has been initialised yet. */
@@ -58,7 +60,10 @@ static GtkWidget *track_progress, *album_progress;
 static GtkWidget *progress_label;
 
 /** The widgets in the main UI */
-static GtkWidget *extract_button, *reread_button, *title_entry, *artist_entry, *track_listview;
+static GtkWidget *extract_button, *reread_button, *title_entry, *artist_entry, *genre_combo, *track_listview;
+
+/** The menuitem in the main menu */
+static GtkWidget *extract_menuitem, *reread_menuitem, *select_all_menuitem, *deselect_all_menuitem;
 
 /** The current TrackDetails being extracted. */
 static TrackDetails *track;
@@ -106,33 +111,28 @@ static Progress before;
 static char*
 build_filename (const TrackDetails *track)
 {
-  char *realfile, *realpath, *filename, *extension, *path;
-  EncoderFormat format;
+  char *realfile, *realpath, *filename, *path;
+  GMAudioProfile *profile;
+  GError *error = NULL;
+
   realpath = filepath_parse_pattern (path_pattern, track);
   realfile = filepath_parse_pattern (file_pattern, track);
-  g_object_get (extractor, "format", &format, NULL);
-  switch (format) {
-  case SJ_FORMAT_VORBIS:
-    extension = ".ogg";
-    break;
-  case SJ_FORMAT_MPEG:
-    extension = ".mp3";
-    break;
-  case SJ_FORMAT_FLAC:
-    extension = ".flac";
-    break;
-  case SJ_FORMAT_WAVE:
-    extension = ".wav";
-    break;
-  default:
-    g_free (realpath);
-    g_free (realfile);
-    g_return_val_if_reached (NULL);
-  }
-  filename = g_strconcat (realfile, extension, NULL);
+  g_object_get (extractor, "profile", &profile, NULL);
+  filename = g_strdup_printf("%s.%s", realfile, 
+			     gm_audio_profile_get_extension(profile));
   path = g_build_filename (base_path, realpath, filename, NULL);
   g_free (realpath);
-  realpath = g_filename_from_utf8 (path, -1, NULL, NULL, NULL);
+  realpath = g_filename_from_utf8 (path, -1, NULL, NULL, &error);
+  if (error) {
+    const char *charset;
+
+    charset = g_getenv("G_FILENAME_ENCODING");
+    if (!charset) {
+      g_get_charset (&charset);
+    }
+    realpath = g_convert_with_fallback (path, -1, charset, "UTF-8","_", NULL, NULL, NULL);
+    g_error_free (error);
+  }
   g_free (path);
   g_free (realfile);
   g_free (filename);
@@ -158,6 +158,12 @@ cleanup (void)
   gtk_widget_set_sensitive (track_listview, TRUE);
   gtk_widget_set_sensitive (title_entry, TRUE);
   gtk_widget_set_sensitive (artist_entry, TRUE);
+  gtk_widget_set_sensitive (genre_combo, TRUE);
+  /* Enabling the Menuitem */ 
+  gtk_widget_set_sensitive (extract_menuitem, TRUE);
+  gtk_widget_set_sensitive (reread_menuitem, TRUE);
+  gtk_widget_set_sensitive (select_all_menuitem, TRUE);
+  gtk_widget_set_sensitive (deselect_all_menuitem, TRUE);
 }
 
 /**
@@ -170,6 +176,8 @@ check_for_file (const char* filename)
   struct stat stats;
   int ret;
   GtkWidget *dialog;
+  char *utf8_filename;
+  
   ret = stat (filename, &stats);
   if (ret == -1) {
     if (errno == ENOENT) {
@@ -184,11 +192,13 @@ check_for_file (const char* filename)
   }
   /* Otherwise the file exists and is large, ask user if they really
      want to overwrite it */
+  utf8_filename = g_filename_to_utf8 (filename, -1, NULL, NULL, NULL);
   dialog = gtk_message_dialog_new (GTK_WINDOW (main_window), GTK_DIALOG_MODAL,
                                    GTK_MESSAGE_QUESTION,
                                    GTK_BUTTONS_NONE,
                                    _("A file called '%s' exists, size %lu kB.\nDo you want to skip this track or overwrite it?"),
-                                   filename, (unsigned long)(stats.st_size / 1000));
+                                   utf8_filename, (unsigned long)(stats.st_size / 1000));
+  g_free (utf8_filename);
   gtk_dialog_add_button (GTK_DIALOG (dialog), _("_Skip"), GTK_RESPONSE_CANCEL);
   gtk_dialog_add_button (GTK_DIALOG (dialog), _("_Overwrite"), GTK_RESPONSE_ACCEPT);
   gtk_dialog_set_default_response (GTK_DIALOG (dialog), GTK_RESPONSE_ACCEPT);
@@ -369,16 +379,17 @@ on_progress_cb (SjExtractor *extractor, const int seconds, gpointer data)
       gettimeofday(&before.time, NULL);
     } else {
       struct timeval time;
-      int ripped, taken;
+      int taken;
       float speed;
 
       gettimeofday(&time, NULL);
-      ripped = current_duration + seconds - before.seconds;
       taken = time.tv_sec + (time.tv_usec / 1000000.0)
-        - (before.time.tv_sec + (before.time.tv_usec / 1000000.0));
-      speed = (float) ripped / (float) taken;
+        - (before.time.tv_sec + (before.time.tv_usec / 1000000.0));	
       if (taken >= 4) {
-        update_speed_progress (extractor, speed, (int) ((total_duration - current_duration + seconds) / speed));
+	before.taken += taken;
+	before.ripped += current_duration + seconds - before.seconds;
+        speed = (float) before.ripped / (float) before.taken;
+        update_speed_progress (extractor, speed, (int) ((total_duration - current_duration - seconds) / speed));
         before.seconds = current_duration + seconds;
         gettimeofday(&before.time, NULL);
       }
@@ -426,6 +437,15 @@ show_finished_dialog (void)
 {
   GtkWidget *dialog;
   int result;
+  char *base;
+
+  base = NULL;
+  /* Find the deepest common directory. */
+  g_list_foreach (paths, (GFunc)base_finder, &base);
+
+  cleanup ();
+  extracting = FALSE;
+
   dialog = gtk_message_dialog_new (GTK_WINDOW (main_window),
                                    GTK_DIALOG_DESTROY_WITH_PARENT,
                                    GTK_MESSAGE_INFO,
@@ -442,22 +462,18 @@ show_finished_dialog (void)
                           GTK_STOCK_OPEN, 1,
                           GTK_STOCK_CLOSE, GTK_RESPONSE_CLOSE,
                           NULL);
+  gtk_dialog_set_default_response (GTK_DIALOG (dialog), GTK_RESPONSE_CLOSE);
   result = gtk_dialog_run (GTK_DIALOG (dialog));
   gtk_widget_destroy (dialog);
   if (result == 1) {
-    char *base, *command;
-    base = NULL;
-    /* Find the deepest common directory. */
-    g_list_foreach (paths, (GFunc)base_finder, &base);
-    /* Construct a Nautilus command line */
-    /* TODO: don't think I need --no-desktop here */
-    command = g_strdup_printf ("nautilus --no-desktop \'%s\'", base);
+    char *command;
+    command = g_strdup_printf ("gnome-open \"%s\"", base);
     g_spawn_command_line_async (command, NULL);
-    g_free (base);
     g_free (command);
   } else if (result == 2) {
     eject_cdrom (device, GTK_WINDOW (main_window));
   }
+  g_free (base);
 }
 
 /**
@@ -470,8 +486,6 @@ on_completion_cb (SjExtractor *extractor, gpointer data)
   if (pending == NULL) {
     gtk_widget_hide (progress_dialog);
     show_finished_dialog ();
-    cleanup ();
-    extracting = FALSE;
     if (autostart) {
       gtk_main_quit ();
     }
@@ -559,8 +573,15 @@ on_extract_activate (GtkWidget *button, gpointer user_data)
     reread_button = glade_xml_get_widget (glade, "reread_button");
     title_entry = glade_xml_get_widget (glade, "title_entry");
     artist_entry = glade_xml_get_widget (glade, "artist_entry");
+    genre_combo = glade_xml_get_widget (glade, "genre_combo");
     track_listview = glade_xml_get_widget (glade, "track_listview");
-    /* TODO : this callback should be in the glade file */
+    
+	extract_menuitem = glade_xml_get_widget (glade, "extract_menuitem");
+	reread_menuitem = glade_xml_get_widget (glade, "re-read");
+	select_all_menuitem = glade_xml_get_widget (glade, "select_all");
+	deselect_all_menuitem = glade_xml_get_widget (glade, "deselect_all");
+	
+	/* TODO : this callback should be in the glade file */
     g_signal_connect (progress_dialog, "delete-event", G_CALLBACK (on_progress_cancel_clicked), NULL);
 
     initialised = TRUE;
@@ -577,8 +598,15 @@ on_extract_activate (GtkWidget *button, gpointer user_data)
   gtk_widget_set_sensitive (reread_button, FALSE);
   gtk_widget_set_sensitive (title_entry, FALSE);
   gtk_widget_set_sensitive (artist_entry, FALSE);
+  gtk_widget_set_sensitive (genre_combo, FALSE);
   gtk_widget_set_sensitive (track_listview, FALSE);
 
+  /* Disable the menuitems in the main menu*/	
+  gtk_widget_set_sensitive (extract_menuitem, FALSE);
+  gtk_widget_set_sensitive (reread_menuitem, FALSE);
+  gtk_widget_set_sensitive (select_all_menuitem, FALSE);
+  gtk_widget_set_sensitive (deselect_all_menuitem, FALSE);
+  
   /* Start the extracting */
   extracting = TRUE;
   pop_and_extract ();
@@ -606,7 +634,7 @@ sanitize_path (char* s)
   g_strdelimit (s, "/", ' ');
   if (strip_chars) {
     /* Mangle all weird characters to whitespace */
-    g_strdelimit (s, "\\*?&!:\"", ' ');
+    g_strdelimit (s, "\\*?&!:\'\"$()`|>{}", ' ');
     /* Replace all whitespace with underscores */
     /* TODO: I'd like this to compress whitespace aswell */
     g_strdelimit (s, "\t ", '_');
