@@ -31,9 +31,11 @@
 #include <gconf/gconf-client.h>
 #include <libgnome/gnome-help.h>
 #include <libgnomeui/gnome-ui-init.h>
+#include <profiles/gnome-media-profiles.h>
 #include <gst/gst.h>
 
 #include "sj-about.h"
+#include "sj-genres.h"
 #include "sj-metadata.h"
 #include "sj-metadata-musicbrainz.h"
 #include "sj-extractor.h"
@@ -42,9 +44,15 @@
 #include "sj-util.h"
 #include "sj-prefs.h"
 
+gboolean on_delete_event (GtkWidget *widget, GdkEvent *event, gpointer user_data);
+
 static void reread_cd (gboolean ignore_no_media);
 static void update_ui_for_album (AlbumDetails *album);
 
+/* Prototypes for the signal blocking/unblocking in update_ui_for_album */
+void on_title_edit_changed(GtkEditable *widget, gpointer user_data);
+void on_artist_edit_changed(GtkEditable *widget, gpointer user_data);
+
 GladeXML *glade;
 
 SjMetadata *metadata;
@@ -53,7 +61,7 @@ SjExtractor *extractor;
 GConfClient *gconf_client;
 
 GtkWidget *main_window;
-static GtkWidget *title_entry, *artist_entry, *duration_label;
+static GtkWidget *title_entry, *artist_entry, *duration_label, *genre_combo;
 static GtkWidget *track_listview, *extract_button;
 static GtkWidget *status_bar;
 static GtkWidget *extract_menuitem, *select_all_menuitem, *deselect_all_menuitem;
@@ -66,7 +74,8 @@ gboolean eject_finished;
 gboolean extracting = FALSE;
 static gboolean tray_opened = FALSE;
 static guint poll_id = 0;
-
+static gint total_no_of_tracks;
+static gint no_of_tracks_selected;
 static AlbumDetails *current_album;
 
 int autostart = FALSE;
@@ -93,7 +102,17 @@ static void error_on_start (GError *error)
  */
 void on_quit_activate (GtkMenuItem *item, gpointer user_data)
 {
-  gtk_main_quit ();
+  if (on_delete_event (NULL, NULL, NULL) == FALSE) {
+    gtk_main_quit ();
+  }
+}
+
+/**
+ * Destroy signal Callback
+ */
+void on_destroy_signal (GtkMenuItem *item, gpointer user_data)
+{
+   gtk_main_quit ();
 }
 
 /**
@@ -122,7 +141,7 @@ gboolean poll_tray_opened (gpointer data)
   return TRUE;
 }
 
-gboolean on_destroy_event (GtkWidget *widget, GdkEvent *event, gpointer user_data)
+gboolean on_delete_event (GtkWidget *widget, GdkEvent *event, gpointer user_data)
 {
   if (extracting) {
     GtkWidget *dialog;
@@ -137,7 +156,7 @@ gboolean on_destroy_event (GtkWidget *widget, GdkEvent *event, gpointer user_dat
     response = gtk_dialog_run (GTK_DIALOG (dialog));
     gtk_widget_destroy (dialog);
     
-    if (response != GTK_RESPONSE_ACCEPT) {
+    if (response == GTK_RESPONSE_ACCEPT) {
       if (poll_id > 0) {
         g_source_remove (poll_id);
       }
@@ -163,6 +182,9 @@ void on_select_all_activate (GtkMenuItem *item, gpointer user_data)
   gtk_tree_model_foreach (GTK_TREE_MODEL (track_store), select_all_foreach_cb, GINT_TO_POINTER (TRUE));
   gtk_widget_set_sensitive (extract_button, TRUE);
   gtk_widget_set_sensitive (extract_menuitem, TRUE);
+  gtk_widget_set_sensitive (select_all_menuitem, FALSE);
+  gtk_widget_set_sensitive (deselect_all_menuitem, TRUE);
+  no_of_tracks_selected = total_no_of_tracks;
 }
 
 void on_deselect_all_activate (GtkMenuItem *item, gpointer user_data)
@@ -170,6 +192,9 @@ void on_deselect_all_activate (GtkMenuItem *item, gpointer user_data)
   gtk_tree_model_foreach (GTK_TREE_MODEL (track_store), select_all_foreach_cb, GINT_TO_POINTER (FALSE));
   gtk_widget_set_sensitive (extract_button, FALSE);
   gtk_widget_set_sensitive (extract_menuitem, FALSE);
+  gtk_widget_set_sensitive (deselect_all_menuitem, FALSE);
+  gtk_widget_set_sensitive (select_all_menuitem,TRUE);
+  no_of_tracks_selected = 0;
 }
 
 /**
@@ -202,28 +227,38 @@ static void update_ui_for_album (AlbumDetails *album)
   GList *l;
   int album_duration = 0;
   char* duration_text;
+  total_no_of_tracks=0;
 
   if (album == NULL) {
     gtk_list_store_clear (track_store);
     gtk_entry_set_text (GTK_ENTRY (title_entry), "");
     gtk_entry_set_text (GTK_ENTRY (artist_entry), "");
     gtk_label_set_text (GTK_LABEL (duration_label), "");
+    gtk_combo_box_set_active (GTK_COMBO_BOX (genre_combo), -1);
     gtk_widget_set_sensitive (title_entry, FALSE);
     gtk_widget_set_sensitive (artist_entry, FALSE);
+    gtk_widget_set_sensitive (genre_combo, FALSE);
     gtk_widget_set_sensitive (extract_button, FALSE);
     gtk_widget_set_sensitive (extract_menuitem, FALSE);
     gtk_widget_set_sensitive (select_all_menuitem, FALSE);
-    gtk_widget_set_sensitive (deselect_all_menuitem, FALSE);
+    gtk_widget_set_sensitive (deselect_all_menuitem, TRUE);
   } else {
     gtk_list_store_clear (track_store);
 
+    g_signal_handlers_block_by_func (title_entry, on_title_edit_changed, NULL);
+    g_signal_handlers_block_by_func (artist_entry, on_artist_edit_changed, NULL);
     gtk_entry_set_text (GTK_ENTRY (title_entry), album->title);
     gtk_entry_set_text (GTK_ENTRY (artist_entry), album->artist);
+    g_signal_handlers_unblock_by_func (title_entry, on_title_edit_changed, NULL);
+    g_signal_handlers_unblock_by_func (artist_entry, on_artist_edit_changed, NULL);
+
+    gtk_combo_box_set_active (GTK_COMBO_BOX (genre_combo), -1);
     gtk_widget_set_sensitive (title_entry, TRUE);
     gtk_widget_set_sensitive (artist_entry, TRUE);
+    gtk_widget_set_sensitive (genre_combo, TRUE);
     gtk_widget_set_sensitive (extract_button, TRUE);
     gtk_widget_set_sensitive (extract_menuitem, TRUE);
-    gtk_widget_set_sensitive (select_all_menuitem, TRUE);
+    gtk_widget_set_sensitive (select_all_menuitem, FALSE);
     gtk_widget_set_sensitive (deselect_all_menuitem, TRUE);
     
     for (l = album->tracks; l; l=g_list_next (l)) {
@@ -239,7 +274,10 @@ static void update_ui_for_album (AlbumDetails *album)
                           COLUMN_DURATION, track->duration,
                           COLUMN_DETAILS, track,
                           -1);
+     total_no_of_tracks++; 
     }
+no_of_tracks_selected=total_no_of_tracks;
+
     /* Some albums don't have track durations :( */
     if (album_duration) {
       duration_text = g_strdup_printf("%d:%02d", album_duration / 60, album_duration % 60);
@@ -589,69 +627,34 @@ void device_changed_cb (GConfClient *client, guint cnxn_id, GConfEntry *entry, g
   }
 }
 
-/**
- * The /apps/sound-juicer/format key has changed.
- */
-void format_changed_cb (GConfClient *client, guint cnxn_id, GConfEntry *entry, gpointer user_data)
+void profile_changed_cb (GConfClient *client, guint cnxn_id, GConfEntry *entry, gpointer user_data)
 {
-  char* value;
-  GEnumValue* enumvalue;
-  int i;
-  int format;
-  gboolean any_support = FALSE;
-  gboolean supported[SJ_NUMBER_FORMATS];
-
-  g_assert (strcmp (entry->key, GCONF_FORMAT) == 0);
+  GMAudioProfile *profile;
+  
+  g_assert (strcmp (entry->key, GCONF_AUDIO_PROFILE) == 0);
   if (!entry->value) return;
-  value = g_ascii_strup (gconf_value_get_string (entry->value), -1);
-  /* TODO: this line is pretty convoluted */
-  enumvalue = g_enum_get_value_by_name (g_type_class_peek (encoding_format_get_type()), value);
-  if (enumvalue == NULL) {
-    /* g_warning (_("Unknown format %s"), value); */
-    format = 0;
-  } else {
-    format = enumvalue->value;
-  }
-  g_free (value);
+  profile = gm_audio_profile_lookup(gconf_value_get_string (entry->value));
+  if (profile != NULL)
+    g_object_set(extractor, "profile", profile, NULL);
 
-  g_object_set (extractor, "format", format, NULL);
-
-  for (i = 0; i < SJ_NUMBER_FORMATS; i++) {
-    supported[i] = sj_extractor_supports_format (i);
-    if (supported[i]) {
-       any_support = TRUE;
-    }
-  }
-
-  if (any_support == FALSE) {
-    GError *err = NULL;
-
-    g_set_error (&err, SJ_ERROR, SJ_ERROR_INTERNAL_ERROR,
-		    _("Cannot find any suitable encoders"));
-    error_on_start (err);
-    exit (1);
-  }
-
-  /* Popup the prefs for the user to change the encoder */
-  for (i = 0; i <= SJ_NUMBER_FORMATS; i++) {
-    if (supported[i] == FALSE && format == i) {
-      GtkWidget *dialog;
-      int response;
-
-      dialog = gtk_message_dialog_new (GTK_WINDOW (main_window),
-		      		       GTK_DIALOG_MODAL,
-				       GTK_MESSAGE_QUESTION,
-				       GTK_BUTTONS_NONE,
-				       _("The currently selected encoder is not available on your installation."));
-      gtk_dialog_add_button (GTK_DIALOG (dialog), "gtk-quit", GTK_RESPONSE_REJECT);
-      gtk_dialog_add_button (GTK_DIALOG (dialog), _("_Change the encoder"), GTK_RESPONSE_ACCEPT);
-      response = gtk_dialog_run (GTK_DIALOG (dialog));
-      if (response == GTK_RESPONSE_ACCEPT) {
-        gtk_widget_destroy (dialog);
-        on_edit_preferences_cb (NULL, NULL);
-      } else {
-        gtk_main_quit ();
-      }
+  if (profile == NULL || !sj_extractor_supports_profile(profile)) {
+    GtkWidget *dialog;
+    int response;
+    
+    dialog = gtk_message_dialog_new (GTK_WINDOW (main_window),
+				     GTK_DIALOG_MODAL,
+				     GTK_MESSAGE_QUESTION,
+				     GTK_BUTTONS_NONE,
+				     _("The currently selected audio profile is not available on your installation."));
+    gtk_dialog_add_button (GTK_DIALOG (dialog), "gtk-quit", GTK_RESPONSE_REJECT);
+    gtk_dialog_add_button (GTK_DIALOG (dialog), _("_Change Profile"), GTK_RESPONSE_ACCEPT);
+    response = gtk_dialog_run (GTK_DIALOG (dialog));
+    if (response == GTK_RESPONSE_ACCEPT) {
+      gtk_widget_destroy (dialog);
+      on_edit_preferences_cb (NULL, NULL);
+    } else {
+      /* Can't use gtk_main_quit here, we may be outside the main loop */
+      exit(0);
     }
   }
 }
@@ -662,8 +665,7 @@ void format_changed_cb (GConfClient *client, guint cnxn_id, GConfEntry *entry, g
 void
 http_proxy_setup (GConfClient *client)
 {
-  if (!gconf_client_get_bool (client, GCONF_HTTP_PROXY_ENABLE, NULL))
-  {
+  if (!gconf_client_get_bool (client, GCONF_HTTP_PROXY_ENABLE, NULL)) {
     sj_metadata_set_proxy (metadata, NULL);
   } else {
     const char *host;
@@ -749,18 +751,31 @@ static void on_extract_toggled (GtkCellRendererToggle *cellrenderertoggle,
   /* extract is the old state here, so toggle */
   extract = !extract;
   gtk_list_store_set (track_store, &iter, COLUMN_EXTRACT, extract, -1);
-
+    
   /* Update the Extract buttons */
   if (extract) {
     /* If true, then we can extract */
     gtk_widget_set_sensitive (extract_button, TRUE);
     gtk_widget_set_sensitive (extract_menuitem, TRUE);
+    no_of_tracks_selected++;
   } else {
     /* Reuse the boolean extract */
     extract = FALSE;
     gtk_tree_model_foreach (GTK_TREE_MODEL (track_store), (GtkTreeModelForeachFunc)extract_available_foreach, &extract);
     gtk_widget_set_sensitive (extract_button, extract);
-    gtk_widget_set_sensitive (extract_menuitem, extract);    
+    gtk_widget_set_sensitive (extract_menuitem, extract);   
+    no_of_tracks_selected--; 
+  }
+  /* Enable and disable the Select/Deselect All buttons */
+  if (no_of_tracks_selected == total_no_of_tracks) {
+    gtk_widget_set_sensitive(deselect_all_menuitem, TRUE);
+    gtk_widget_set_sensitive(select_all_menuitem, FALSE);
+  } else if (no_of_tracks_selected == 0) {
+    gtk_widget_set_sensitive(deselect_all_menuitem, FALSE);
+    gtk_widget_set_sensitive(select_all_menuitem, TRUE);
+  } else {
+    gtk_widget_set_sensitive(select_all_menuitem, TRUE);
+    gtk_widget_set_sensitive(deselect_all_menuitem, TRUE);
   }
 }
 
@@ -826,11 +841,26 @@ void on_artist_edit_changed(GtkEditable *widget, gpointer user_data) {
     gtk_tree_model_get (GTK_TREE_MODEL (track_store), &iter, COLUMN_DETAILS, &track, -1);
     g_free (track->artist);
     track->artist = g_strdup (current_album->artist);
-    /* TODO: should I g_strdup(->artist) here or does _set do that? */
     gtk_list_store_set (track_store, &iter, COLUMN_ARTIST, track->artist, -1);
   } while (gtk_tree_model_iter_next (GTK_TREE_MODEL(track_store), &iter));
 }
 
+static gboolean genre_foreach (GtkTreeModel *model, GtkTreePath *path, GtkTreeIter *iter, gpointer data) {
+  TrackDetails *track;
+  gtk_tree_model_get (model, iter, COLUMN_DETAILS, &track, -1);
+  track->genre = GPOINTER_TO_INT(data);
+  return FALSE;
+}
+
+void on_genre_combo_changed(GtkComboBox *combo, gpointer user_data) {
+  GtkTreeIter iter;
+  if (gtk_combo_box_get_active_iter (combo, &iter)) {
+    int num;
+    gtk_tree_model_get (gtk_combo_box_get_model (combo), &iter, 0, &num, -1);
+    gtk_tree_model_foreach (GTK_TREE_MODEL (track_store), (GtkTreeModelForeachFunc)genre_foreach, GINT_TO_POINTER(num));
+  }
+}
+
 void on_contents_activate(GtkWidget *button, gpointer user_data) {
   GError *error = NULL;
 
@@ -853,6 +883,44 @@ void on_contents_activate(GtkWidget *button, gpointer user_data) {
   }
 }
 
+static int genre_cmp (gconstpointer a, gconstpointer b)
+{
+  GenreMap *p1, *p2;
+  p1 = (GenreMap *)a;
+  p2 = (GenreMap *)b;
+  return strcmp (sj_genre_name (p1->canonical), sj_genre_name (p2->canonical));
+}
+
+static GtkTreeModel* populate_genre_list(void) {
+  GtkListStore *store;
+  GenreMap *p;
+  GList *genres;
+  GList *g;
+
+  store = gtk_list_store_new (2, G_TYPE_INT, G_TYPE_STRING);
+  p = genremap;
+  
+  genres = NULL;
+  while (p->genre != LAST_GENRE) {
+    if (p->genre == p->canonical) {
+      genres = g_list_insert_sorted (genres, (gpointer)p, genre_cmp);
+    }
+    ++p;
+  }
+
+  g = genres;
+  while (g != NULL) {
+    GtkTreeIter iter;
+    p = (GenreMap *)g->data;
+    gtk_list_store_append (store, &iter);
+    gtk_list_store_set (store, &iter, 0, p->canonical, 1, sj_genre_name (p->canonical), -1);
+    g = g->next;
+  }
+  g_list_free (genres);
+
+  return GTK_TREE_MODEL (store);
+}
+
 int main (int argc, char **argv)
 {
   GError *error = NULL;
@@ -883,13 +951,6 @@ int main (int argc, char **argv)
   }
   g_signal_connect (metadata, "metadata", G_CALLBACK (metadata_cb), NULL);
 
-  extractor = SJ_EXTRACTOR (sj_extractor_new());
-  error = sj_extractor_get_new_error (extractor);
-  if (error) {
-    error_on_start (error);
-    exit (1);
-  }
-
   gconf_client = gconf_client_get_default ();
   if (gconf_client == NULL) {
     g_warning (_("Could not create GConf client.\n"));
@@ -901,7 +962,7 @@ int main (int argc, char **argv)
   gconf_client_notify_add (gconf_client, GCONF_EJECT, eject_changed_cb, NULL, NULL, NULL);
   gconf_client_notify_add (gconf_client, GCONF_BASEPATH, basepath_changed_cb, NULL, NULL, NULL);
   gconf_client_notify_add (gconf_client, GCONF_STRIP, strip_changed_cb, NULL, NULL, NULL);
-  gconf_client_notify_add (gconf_client, GCONF_FORMAT, format_changed_cb, NULL, NULL, NULL);
+  gconf_client_notify_add (gconf_client, GCONF_AUDIO_PROFILE, profile_changed_cb, NULL, NULL, NULL);
   gconf_client_notify_add (gconf_client, GCONF_PARANOIA, paranoia_changed_cb, NULL, NULL, NULL);
   gconf_client_notify_add (gconf_client, GCONF_PATH_PATTERN, path_pattern_changed_cb, NULL, NULL, NULL);
   gconf_client_notify_add (gconf_client, GCONF_FILE_PATTERN, file_pattern_changed_cb, NULL, NULL, NULL);
@@ -910,6 +971,9 @@ int main (int argc, char **argv)
   gconf_client_notify_add (gconf_client, GCONF_HTTP_PROXY, http_proxy_changed_cb, NULL, NULL, NULL);
   gconf_client_notify_add (gconf_client, GCONF_HTTP_PROXY_PORT, http_proxy_port_changed_cb, NULL, NULL, NULL);
 
+  /* init gnome-media-profiles */
+  gnome_media_profiles_init (gconf_client);
+
   glade_init ();
   glade = glade_xml_new (DATADIR"/sound-juicer/sound-juicer.glade", NULL, NULL);
   g_assert (glade != NULL);
@@ -924,10 +988,19 @@ int main (int argc, char **argv)
   title_entry = glade_xml_get_widget (glade, "title_entry");
   artist_entry = glade_xml_get_widget (glade, "artist_entry");
   duration_label = glade_xml_get_widget (glade, "duration_label");
+  genre_combo = glade_xml_get_widget (glade, "genre_combo");
   track_listview = glade_xml_get_widget (glade, "track_listview");
   extract_button = glade_xml_get_widget (glade, "extract_button");
   status_bar = glade_xml_get_widget (glade, "status_bar");
 
+  gtk_combo_box_set_model (GTK_COMBO_BOX (genre_combo), populate_genre_list ());
+  {
+    GtkCellRenderer *renderer;
+    renderer = gtk_cell_renderer_text_new ();
+    gtk_cell_layout_pack_start (GTK_CELL_LAYOUT(genre_combo), renderer, TRUE);
+    gtk_cell_layout_add_attribute(GTK_CELL_LAYOUT(genre_combo), renderer, "text", 1);
+  }
+
   track_store = gtk_list_store_new (COLUMN_TOTAL, G_TYPE_BOOLEAN, G_TYPE_INT, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_INT, G_TYPE_POINTER);
   gtk_tree_view_set_model (GTK_TREE_VIEW (track_listview), GTK_TREE_MODEL (track_store));
   {
@@ -960,6 +1033,7 @@ int main (int argc, char **argv)
                                                        "text", COLUMN_TITLE,
                                                        NULL);
     gtk_tree_view_column_set_resizable (column, TRUE);
+    gtk_tree_view_column_set_expand (column, TRUE);
     gtk_tree_view_append_column (GTK_TREE_VIEW (track_listview), column);
 
     text_renderer = gtk_cell_renderer_text_new ();
@@ -968,6 +1042,7 @@ int main (int argc, char **argv)
                                                        "text", COLUMN_ARTIST,
                                                        NULL);
     gtk_tree_view_column_set_resizable (column, TRUE);
+    gtk_tree_view_column_set_expand (column, TRUE);
     g_signal_connect (text_renderer, "edited", G_CALLBACK (on_cell_edited), GUINT_TO_POINTER (COLUMN_ARTIST));
     g_object_set (G_OBJECT (text_renderer), "editable", TRUE, NULL);
     gtk_tree_view_append_column (GTK_TREE_VIEW (track_listview), column);
@@ -984,11 +1059,18 @@ int main (int argc, char **argv)
   update_ui_for_album (NULL);
   gtk_widget_show_all (main_window);
 
+  extractor = SJ_EXTRACTOR (sj_extractor_new());
+  error = sj_extractor_get_new_error (extractor);
+  if (error) {
+    error_on_start (error);
+    exit (1);
+  }
+
   http_proxy_setup (gconf_client);
   basepath_changed_cb (gconf_client, -1, gconf_client_get_entry (gconf_client, GCONF_BASEPATH, NULL, TRUE, NULL), NULL);
   path_pattern_changed_cb (gconf_client, -1, gconf_client_get_entry (gconf_client, GCONF_PATH_PATTERN, NULL, TRUE, NULL), NULL);
   file_pattern_changed_cb (gconf_client, -1, gconf_client_get_entry (gconf_client, GCONF_FILE_PATTERN, NULL, TRUE, NULL), NULL);
-  format_changed_cb (gconf_client, -1, gconf_client_get_entry (gconf_client, GCONF_FORMAT, NULL, TRUE, NULL), NULL);
+  profile_changed_cb (gconf_client, -1, gconf_client_get_entry (gconf_client, GCONF_AUDIO_PROFILE, NULL, TRUE, NULL), NULL);
   paranoia_changed_cb (gconf_client, -1, gconf_client_get_entry (gconf_client, GCONF_PARANOIA, NULL, TRUE, NULL), NULL);
   strip_changed_cb (gconf_client, -1, gconf_client_get_entry (gconf_client, GCONF_STRIP, NULL, TRUE, NULL), NULL);
   eject_changed_cb (gconf_client, -1, gconf_client_get_entry (gconf_client, GCONF_EJECT, NULL, TRUE, NULL), NULL);
