@@ -20,7 +20,9 @@
 
 #include "sound-juicer.h"
 
+#include <string.h>
 #include <glib-object.h>
+#include <glib/gi18n.h>
 #include <glib/gerror.h>
 #include <glib/glist.h>
 #include <glib/gstrfuncs.h>
@@ -33,6 +35,7 @@
 #include "sj-metadata-musicbrainz.h"
 #include "sj-structures.h"
 #include "sj-error.h"
+#include "sj-genres.h"
 #include "cd-drive.h"
 
 struct SjMetadataMusicbrainzPrivate {
@@ -74,6 +77,8 @@ sj_metadata_musicbrainz_finalize (GObject *object)
 static void
 sj_metadata_musicbrainz_instance_init (GTypeInstance *instance, gpointer g_class)
 {
+  GConfClient *gconf_client;
+  char *server_name = NULL;
   SjMetadataMusicbrainz *self = (SjMetadataMusicbrainz*)instance;
   self->priv = g_new0 (SjMetadataMusicbrainzPrivate, 1);
   self->priv->construct_error = NULL;
@@ -86,6 +91,18 @@ sj_metadata_musicbrainz_instance_init (GTypeInstance *instance, gpointer g_class
     return;
   }
   mb_UseUTF8 (self->priv->mb, TRUE);
+
+  gconf_client = gconf_client_get_default ();
+  server_name = gconf_client_get_string (gconf_client, GCONF_MUSICBRAINZ_SERVER, NULL);
+  if (server_name) {
+    g_strstrip (server_name);
+  }
+  if (server_name && strcmp (server_name, "") != 0) {
+    mb_SetServer (self->priv->mb, server_name, 80);
+    g_free (server_name);
+  }
+  g_object_unref (gconf_client);
+
   /* TODO: optimal setting? mb_SetDepth (self->priv->mb, 1); */
   if (g_getenv("MUSICBRAINZ_DEBUG")) {
     mb_SetDebug (self->priv->mb, TRUE);
@@ -191,6 +208,7 @@ get_offline_track_listing(SjMetadata *metadata, GError **error)
     track->title = g_strdup_printf (_("Track %d"), i);
     track->artist = g_strdup (album->artist);
     track->duration = get_duration_from_sectors (mb_GetResultInt1 (priv->mb, MBE_TOCGetTrackNumSectors, i+1));
+    track->genre = LAST_GENRE;
     album->tracks = g_list_append (album->tracks, track);
     album->number++;
   }
@@ -284,7 +302,7 @@ lookup_cd (SjMetadata *metadata)
   g_return_val_if_fail (priv->cdrom != NULL, NULL);
   priv->error = NULL; /* TODO: hack */
 
-  type = guess_media_type (priv->cdrom);
+  type = cd_drive_get_media_type_from_path (priv->cdrom);
   if (type == CD_MEDIA_TYPE_ERROR) {
     char *msg;
     SjError err;
@@ -317,7 +335,7 @@ lookup_cd (SjMetadata *metadata)
     g_print (_("This CD was not found.\n"));
     priv->albums = get_offline_track_listing (metadata, &(priv->error));
     g_idle_add ((GSourceFunc)fire_signal_idle, metadata);
-    return albums;
+    return priv->albums;
   }
 
   for (i = 1; i <= num_albums; i++) {
@@ -327,9 +345,11 @@ lookup_cd (SjMetadata *metadata)
     mb_Select1(priv->mb, MBS_SelectAlbum, i);
     album = g_new0 (AlbumDetails, 1);
 
-    mb_GetResultData(priv->mb, MBE_AlbumGetAlbumName, data, MB_BUFFER_SIZE);
-    /* TODO: handle failure */
-    album->title = g_strdup (data);
+    if (mb_GetResultData(priv->mb, MBE_AlbumGetAlbumName, data, MB_BUFFER_SIZE)) {
+      album->title = g_strdup (data);
+    } else {
+      album->title = g_strdup (_("Unknown Title"));
+    }
 
     mb_GetResultData (priv->mb, MBE_AlbumGetAlbumArtistId, data, MB_BUFFER_SIZE);
     /* TODO: handle failure */
@@ -337,13 +357,23 @@ lookup_cd (SjMetadata *metadata)
     if (g_ascii_strncasecmp (MBI_VARIOUS_ARTIST_ID, data, 64) == 0) {
       album->artist = g_strdup (_("Various"));
     } else {
-      mb_GetResultData1(priv->mb, MBE_AlbumGetArtistName, data, MB_BUFFER_SIZE, 1);
-      /* TODO: handle failure */
-      album->artist = g_strdup (data);
+      if (data && mb_GetResultData1(priv->mb, MBE_AlbumGetArtistName, data, MB_BUFFER_SIZE, 1)) {
+        album->artist = g_strdup (data);
+      } else {
+        album->artist = g_strdup (_("Unknown Artist"));
+      }
     }
 
     num_tracks = mb_GetResultInt(priv->mb, MBE_AlbumGetNumTracks);
-    /* TODO: handle failure */
+    if (num_tracks < 1) {
+      g_free (album->artist);
+      g_free (album->title);
+      g_free (album);
+      g_print (_("Incomplete metadata for this CD.\n"));
+      priv->albums = get_offline_track_listing (metadata, &(priv->error));
+      g_idle_add ((GSourceFunc)fire_signal_idle, metadata);
+      return priv->albums;
+    }
 
     for (j = 1; j <= num_tracks; j++) {
       TrackDetails *track;
@@ -364,7 +394,9 @@ lookup_cd (SjMetadata *metadata)
       if (mb_GetResultData1(priv->mb, MBE_AlbumGetTrackDuration, data, MB_BUFFER_SIZE, j)) {
         track->duration = atoi (data) / 1000;
       }
-
+      
+      track->genre = LAST_GENRE;
+      
       album->tracks = g_list_append (album->tracks, track);
       album->number++;
     }
