@@ -27,6 +27,7 @@
 #include <glade/glade-xml.h>
 #include <gconf/gconf-client.h>
 #include <libgnome/gnome-help.h>
+#include <profiles/gnome-media-profiles.h>
 #include "sj-extracting.h"
 #include "cd-drive.h"
 #include "bacon-cd-selection.h"
@@ -34,13 +35,14 @@
 extern GladeXML *glade;
 extern GtkWidget *main_window;
 
-/* Store the toggle buttons pertaining to the format */
-static GtkWidget *format_widgets[SJ_NUMBER_FORMATS];
+static GtkWidget *audio_profile;
 static GtkWidget *prefs_dialog;
-static GtkWidget *cd_option, *path_option, *file_option, *basepath_label, *check_strip, *check_eject;
+static GtkWidget *cd_option, *path_option, *file_option, *basepath_fcb, *check_strip, *check_eject;
 static GtkWidget *path_example_label;
 static GList *cdroms = NULL;
 
+#define DEFAULT_AUDIO_PROFILE_NAME "cdlossy"
+
 typedef struct {
   char* name;
   char* pattern;
@@ -82,6 +84,9 @@ gboolean cd_drive_exists (const char *device)
 {
   GList *l;
   if (device == NULL) return FALSE;
+  if (cdroms == NULL) {
+    cdroms = scan_for_cdroms (FALSE, FALSE);
+  }
   for (l = cdroms; l != NULL; l = l->next) {
     if (strcmp (((CDDrive *) (l->data))->device, device) == 0)
       return TRUE;
@@ -94,7 +99,7 @@ gboolean cd_drive_exists (const char *device)
  */
 void prefs_cdrom_changed_cb (GtkWidget *widget, const char* device)
 {
-  gconf_client_set_string (gconf_client, GCONF_DEVICE, device, NULL);
+  gconf_client_set_string (gconf_client, GCONF_DEVICE, device != NULL ? device : "", NULL);
 }
 
 /**
@@ -107,6 +112,17 @@ void prefs_eject_toggled (GtkToggleButton *togglebutton, gpointer user_data)
                          NULL);
 }
 
+void prefs_profile_changed (GtkWidget *widget, gpointer user_data)
+{
+  GMAudioProfile *profile;
+  /* Handle the change being to unselect a profile */
+  if (gtk_combo_box_get_active (GTK_COMBO_BOX (widget)) != -1) {
+    profile = gm_audio_profile_choose_get_active (widget);
+    gconf_client_set_string (gconf_client, GCONF_AUDIO_PROFILE,  
+                             gm_audio_profile_get_id (profile), NULL);
+  }
+}
+
 /**
  * Show the gnome help browser
  */
@@ -134,47 +150,35 @@ void show_help ()
 }
 
 /**
- * Clicked on Browse in the Prefs dialog
+ * Changed folder in the Prefs dialog
  */
-void prefs_browse_clicked (GtkButton *button, gpointer user_data)
+void prefs_base_folder_changed (GtkWidget *chooser, gpointer user_data)
 {
-  GtkWidget *filesel;
-  int res;
-
-  filesel = gtk_file_selection_new(_("Select Output Location"));
-  gtk_file_selection_set_filename (GTK_FILE_SELECTION (filesel), base_path);
-  gtk_widget_show_all (filesel);
-  /* A little hacky, but it works :) */
-  gtk_widget_hide (gtk_widget_get_parent (GTK_FILE_SELECTION(filesel)->file_list));
-  res = gtk_dialog_run (GTK_DIALOG (filesel));
-  if (res == GTK_RESPONSE_OK) {
-    const char *filename;
-    char *path;
-    filename = gtk_file_selection_get_filename(GTK_FILE_SELECTION(filesel));
-    path = g_filename_to_utf8 (filename, -1, NULL, NULL, NULL); /* TODO: GError */
-    gconf_client_set_string (gconf_client, GCONF_BASEPATH, path, NULL);
-    g_free (path);
-  }
-  gtk_widget_destroy (filesel);
+  char *filename, *path;
+  filename = gtk_file_chooser_get_filename (GTK_FILE_CHOOSER (chooser));
+  path = g_filename_to_utf8 (filename, -1, NULL, NULL, NULL); /* TODO: GError */
+  gconf_client_set_string (gconf_client, GCONF_BASEPATH, path, NULL);
+  g_free (path);
+  g_free (filename);
 }
 
-void prefs_path_option_changed (GtkOptionMenu *optionmenu, gpointer user_data)
+void prefs_path_option_changed (GtkComboBox *combo, gpointer user_data)
 {
-  int history;
+  gint active;
   const char* pattern;
-  history = gtk_option_menu_get_history (optionmenu);
-  pattern = path_patterns[history].pattern;
+  active = gtk_combo_box_get_active (combo);
+  pattern = path_patterns[active].pattern;
   if (pattern) {
     gconf_client_set_string (gconf_client, GCONF_PATH_PATTERN, pattern, NULL);
   }
 }
 
-void prefs_file_option_changed (GtkOptionMenu *optionmenu, gpointer user_data)
+void prefs_file_option_changed (GtkComboBox *combo, gpointer user_data)
 {
-  int history;
+  gint active;
   const char* pattern;
-  history = gtk_option_menu_get_history (optionmenu);
-  pattern = file_patterns[history].pattern;
+  active = gtk_combo_box_get_active (combo);
+  pattern = file_patterns[active].pattern;
   if (pattern) {
     gconf_client_set_string (gconf_client, GCONF_FILE_PATTERN, pattern, NULL);
   }
@@ -190,71 +194,32 @@ void prefs_strip_toggled (GtkToggleButton *togglebutton, gpointer user_data)
                          NULL);
 }
 
-/**
- * One of the format toggle buttons in the prefs dialog has been
- * toggled.
- */
-void on_format_toggled (GtkToggleButton *togglebutton,
-                               gpointer user_data)
-{
-  const char* format;
-  if (!gtk_toggle_button_get_active (togglebutton)) {
-    return;
-  }
-  if (GTK_WIDGET (togglebutton) == format_widgets[SJ_FORMAT_VORBIS]) {
-    format = "vorbis";
-  } else if (GTK_WIDGET(togglebutton) == format_widgets[SJ_FORMAT_MPEG]) {
-    format = "mpeg";
-  } else if (GTK_WIDGET(togglebutton) == format_widgets[SJ_FORMAT_FLAC]) {
-    format = "flac";
-  } else if (GTK_WIDGET(togglebutton) == format_widgets[SJ_FORMAT_WAVE]) {
-    format = "wave";
-  } else {
-    return;
-  }
-  gconf_client_set_string (gconf_client, GCONF_FORMAT, format, NULL); /* TODO: GError */
-}
-
 static void device_changed_cb (GConfClient *client, guint cnxn_id, GConfEntry *entry, gpointer user_data)
 {
-  g_assert (strcmp (entry->key, GCONF_DEVICE) == 0);
+  g_return_if_fail (strcmp (entry->key, GCONF_DEVICE) == 0);
 
   if (entry->value == NULL) {
     bacon_cd_selection_set_device (BACON_CD_SELECTION (cd_option),
                                    bacon_cd_selection_get_default_device (BACON_CD_SELECTION (cd_option)));
   } else {
+    g_return_if_fail (entry->value->type == GCONF_VALUE_STRING);
     bacon_cd_selection_set_device (BACON_CD_SELECTION (cd_option),
                                    gconf_value_get_string (entry->value));
   }
 }
 
-static void format_changed_cb (GConfClient *client, guint cnxn_id, GConfEntry *entry, gpointer user_data)
+static void audio_profile_changed_cb (GConfClient *client, guint cnxn_id, GConfEntry *entry, gpointer user_data)
 {
-  char* value;
-  GEnumValue* enumvalue;
-  int format;
-  int i;
-  g_assert (strcmp (entry->key, GCONF_FORMAT) == 0);
+  const char *value;
+  g_return_if_fail (strcmp (entry->key, GCONF_AUDIO_PROFILE) == 0);
   if (!entry->value) return;
-  value = g_ascii_strup (gconf_value_get_string (entry->value), -1);
-  /* TODO: this line is pretty convoluted */
-  enumvalue = g_enum_get_value_by_name (g_type_class_peek (encoding_format_get_type()), value);
-  /* Fallback on vorbis if it's not found */
-  if (enumvalue == NULL) {
-    /* g_warning (_("Unknown format %s"), value); */
-    format = 0;
+  value = gconf_value_get_string (entry->value);
+  
+  /* If the value is empty, unset the combo. Otherwise try and set it. */
+  if (strcmp (value, "") == 0) {
+    gtk_combo_box_set_active (GTK_COMBO_BOX (audio_profile), -1);
   } else {
-    format = enumvalue->value;
-  }
-  g_free (value);
-
-  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (format_widgets[format]), TRUE);
-
-  for (i = 0; i < SJ_NUMBER_FORMATS; i++) {
-    gboolean supported;
-
-    supported = sj_extractor_supports_format (i);
-    gtk_widget_set_sensitive (format_widgets[i], supported);
+    gm_audio_profile_choose_set_active (audio_profile, value);
   }
 }
 
@@ -268,22 +233,26 @@ static void basepath_changed_cb (GConfClient *client, guint cnxn_id, GConfEntry
    * into sj-utils?
    */
   const char* base_path;
-  g_assert (strcmp (entry->key, GCONF_BASEPATH) == 0);
+  g_return_if_fail (strcmp (entry->key, GCONF_BASEPATH) == 0);
+
   if (entry->value == NULL) {
-    base_path = g_get_home_dir ();
+    gtk_file_chooser_set_current_folder (GTK_FILE_CHOOSER (basepath_fcb), g_get_home_dir());
   } else {
+    g_return_if_fail (entry->value->type == GCONF_VALUE_STRING);
     base_path = gconf_value_get_string (entry->value);
+    if (strcmp (gtk_file_chooser_get_current_folder (GTK_FILE_CHOOSER (basepath_fcb)), base_path) != 0) { 
+           gtk_file_chooser_set_current_folder (GTK_FILE_CHOOSER (basepath_fcb), base_path);
+    }
   }
-  /* TODO: sanity check the path somewhat */
-  /* TODO: use an ellipsising label? */
-  gtk_label_set_text (GTK_LABEL (basepath_label), base_path);
 }
 
 static void strip_changed_cb (GConfClient *client, guint cnxn_id, GConfEntry *entry, gpointer user_data)
 {
   gboolean value = FALSE;
-  g_assert (strcmp (entry->key, GCONF_STRIP) == 0);
+  g_return_if_fail (strcmp (entry->key, GCONF_STRIP) == 0);
+
   if (entry->value != NULL) {
+    g_return_if_fail (entry->value->type == GCONF_VALUE_BOOL);
     value = gconf_value_get_bool (entry->value);
   }
   gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (check_strip), value);
@@ -292,8 +261,10 @@ static void strip_changed_cb (GConfClient *client, guint cnxn_id, GConfEntry *en
 static void eject_changed_cb (GConfClient *client, guint cnxn_id, GConfEntry *entry, gpointer user_data)
 {
   gboolean value = FALSE;
-  g_assert (strcmp (entry->key, GCONF_EJECT) == 0);
+  g_return_if_fail (strcmp (entry->key, GCONF_EJECT) == 0);
+
   if (entry->value != NULL) {
+    g_return_if_fail (entry->value->type == GCONF_VALUE_BOOL);
     value = gconf_value_get_bool (entry->value);
   }
   gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (check_eject), value);
@@ -302,7 +273,7 @@ static void eject_changed_cb (GConfClient *client, guint cnxn_id, GConfEntry *en
 static void pattern_label_update (void)
 {
   char *file_pattern, *path_pattern;
-  char *file_value, *path_value, *example;
+  char *file_value, *path_value, *example, *format;
   gboolean strip;
   static AlbumDetails sample_album = {
     N_("Album Title"),
@@ -315,7 +286,8 @@ static void pattern_label_update (void)
     9,
     N_("Track Title"),
     N_("Track Artist"),
-    60
+    60,
+    LAST_GENRE
   };
   /* Magic to i18n-ize the sample strings. */
   static gboolean been_here = FALSE;
@@ -346,25 +318,30 @@ static void pattern_label_update (void)
   g_free (path_value);
   g_free (path_pattern);
 
-  gtk_label_set_text (GTK_LABEL (path_example_label), example+1);
+  format = g_strconcat ("<i><small>", example, "</small></i>", NULL);
   g_free (example);
+  
+  gtk_label_set_markup (GTK_LABEL (path_example_label), format);
+  g_free (format);
 }
 
 static void path_pattern_changed_cb (GConfClient *client, guint cnxn_id, GConfEntry *entry, gpointer user_data)
 {
   char *value;
   int i = 0;
-  g_assert (strcmp (entry->key, GCONF_PATH_PATTERN) == 0);
+  g_return_if_fail (strcmp (entry->key, GCONF_PATH_PATTERN) == 0);
+
   if (entry->value == NULL) {
     value = g_strdup (path_patterns[0].pattern);
-  } else {
+  } else if (entry->value->type == GCONF_VALUE_STRING) {
     value = g_strdup (gconf_value_get_string (entry->value));
-  }
+  } else
+    return;
   while (path_patterns[i].pattern && strcmp(path_patterns[i].pattern, value) != 0) {
     i++;
   }
   g_free (value);
-  gtk_option_menu_set_history (GTK_OPTION_MENU (path_option), i);
+  gtk_combo_box_set_active (GTK_COMBO_BOX (path_option), i);
   pattern_label_update ();
 }
 
@@ -372,35 +349,38 @@ static void file_pattern_changed_cb (GConfClient *client, guint cnxn_id, GConfEn
 {
   char *value;
   int i = 0;
-  g_assert (strcmp (entry->key, GCONF_FILE_PATTERN) == 0);
+  
+  g_return_if_fail (strcmp (entry->key, GCONF_FILE_PATTERN) == 0);
+  
   if (entry->value == NULL) {
     value = g_strdup (file_patterns[0].pattern);
-  } else {
+  } else if (entry->value->type == GCONF_VALUE_STRING) {
     value = g_strdup (gconf_value_get_string (entry->value));
-  }
+  } else
+    return;
   while (file_patterns[i].pattern && strcmp(file_patterns[i].pattern, value) != 0) {
     i++;
   }
   g_free (value);
-  gtk_option_menu_set_history (GTK_OPTION_MENU (file_option), i);
+  gtk_combo_box_set_active (GTK_COMBO_BOX (file_option), i);
   pattern_label_update ();
 }
 
 /**
- * Given a FilePattern array, generate a menu of them items
+ * Given a FilePattern array, populate the combo box.
  */
-static GtkWidget *generate_pattern_menu (FilePattern *patterns)
+static void populate_pattern_combo (GtkComboBox *combo, FilePattern *patterns)
 {
+  GtkListStore *store;
   int i;
-  GtkWidget *menu, *item;
-  menu = gtk_menu_new ();
   
+  store = gtk_list_store_new (2, G_TYPE_STRING, G_TYPE_STRING);
   for (i = 0; patterns[i].pattern; ++i) {
-    item = gtk_menu_item_new_with_label (_(patterns[i].name));
-    g_object_set_data (G_OBJECT (item), "pattern", patterns[i].pattern);
-    gtk_menu_shell_append (GTK_MENU_SHELL (menu), item);
+    GtkTreeIter iter;
+    gtk_list_store_append (store, &iter);
+    gtk_list_store_set (store, &iter, 0, _(patterns[i].name), 1, patterns[i].pattern, -1);
   }
-  return menu;
+  gtk_combo_box_set_model (combo, GTK_TREE_MODEL (store));
 }
 
 /**
@@ -417,26 +397,23 @@ void on_edit_preferences_cb (GtkMenuItem *item, gpointer user_data)
 
 
     cd_option = glade_xml_get_widget (glade, "cd_option");
-    basepath_label = glade_xml_get_widget (glade, "path_label");
+    basepath_fcb = glade_xml_get_widget (glade, "path_chooser");
     path_option = glade_xml_get_widget (glade, "path_option");
     file_option = glade_xml_get_widget (glade, "file_option");
-    format_widgets[SJ_FORMAT_VORBIS] = glade_xml_get_widget (glade, "format_vorbis");
-    format_widgets[SJ_FORMAT_MPEG] = glade_xml_get_widget (glade, "format_mpeg");
-    format_widgets[SJ_FORMAT_FLAC] = glade_xml_get_widget (glade, "format_flac");
-    format_widgets[SJ_FORMAT_WAVE] = glade_xml_get_widget (glade, "format_wave");
+    audio_profile = glade_xml_get_widget (glade, "audio_profile");
     check_strip = glade_xml_get_widget (glade, "check_strip");
     check_eject = glade_xml_get_widget (glade, "check_eject");
     path_example_label = glade_xml_get_widget (glade, "path_example_label");
 
-    gtk_option_menu_set_menu (GTK_OPTION_MENU (path_option), generate_pattern_menu (path_patterns));
+    populate_pattern_combo (GTK_COMBO_BOX (path_option), path_patterns);
     g_signal_connect (path_option, "changed", G_CALLBACK (prefs_path_option_changed), NULL);
-    gtk_option_menu_set_menu (GTK_OPTION_MENU (file_option), generate_pattern_menu (file_patterns));
+    populate_pattern_combo (GTK_COMBO_BOX (file_option), file_patterns);
     g_signal_connect (file_option, "changed", G_CALLBACK (prefs_file_option_changed), NULL);
 
     /* Connect to GConf to update the UI */
     gconf_client_notify_add (gconf_client, GCONF_DEVICE, device_changed_cb, NULL, NULL, NULL);
     gconf_client_notify_add (gconf_client, GCONF_BASEPATH, basepath_changed_cb, NULL, NULL, NULL);
-    gconf_client_notify_add (gconf_client, GCONF_FORMAT, format_changed_cb, NULL, NULL, NULL);
+    gconf_client_notify_add (gconf_client, GCONF_AUDIO_PROFILE, audio_profile_changed_cb, NULL, NULL, NULL);
     gconf_client_notify_add (gconf_client, GCONF_STRIP, strip_changed_cb, NULL, NULL, NULL);
     gconf_client_notify_add (gconf_client, GCONF_EJECT, eject_changed_cb, NULL, NULL, NULL);
     gconf_client_notify_add (gconf_client, GCONF_PATH_PATTERN, path_pattern_changed_cb, NULL, NULL, NULL);
@@ -444,7 +421,7 @@ void on_edit_preferences_cb (GtkMenuItem *item, gpointer user_data)
   }
   basepath_changed_cb (gconf_client, -1, gconf_client_get_entry (gconf_client, GCONF_BASEPATH, NULL, TRUE, NULL), NULL);
   device_changed_cb (gconf_client, -1, gconf_client_get_entry (gconf_client, GCONF_DEVICE, NULL, TRUE, NULL), NULL);
-  format_changed_cb (gconf_client, -1, gconf_client_get_entry (gconf_client, GCONF_FORMAT, NULL, TRUE, NULL), NULL);
+  audio_profile_changed_cb (gconf_client, -1, gconf_client_get_entry (gconf_client, GCONF_AUDIO_PROFILE, NULL, TRUE, NULL), NULL);
   strip_changed_cb (gconf_client, -1, gconf_client_get_entry (gconf_client, GCONF_STRIP, NULL, TRUE, NULL), NULL);
   eject_changed_cb (gconf_client, -1, gconf_client_get_entry (gconf_client, GCONF_EJECT, NULL, TRUE, NULL), NULL);
   file_pattern_changed_cb (gconf_client, -1, gconf_client_get_entry (gconf_client, GCONF_FILE_PATTERN, NULL, TRUE, NULL), NULL);
@@ -463,3 +440,8 @@ void on_edit_preferences_cb (GtkMenuItem *item, gpointer user_data)
       
   gtk_widget_hide (prefs_dialog);
 }
+
+GtkWidget* create_file_chooser_button(void)
+{
+  return gtk_file_chooser_button_new (_("Select Folder"), GTK_FILE_CHOOSER_ACTION_SELECT_FOLDER);
+}
